// os9guess.c

// Attempt to determine the format of a os9 disk. This is done
// by issuing fdrawcmd commands attempting to read 256 bytes of
// LSN0. If the read is successful, we extract the counts from
// it.

// Bob van der Poel   98/06/04
// bvpoel@uniserve.com

#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <getopt.h>

char *sfile="guess.output";
int nosync=0;

void usage(void);
void testoutput(int drive, int track, int sector, int rate, int density);
void docmd(int drive, int track, int sector, int rate, int density, char *dev);


int
main(int argc, char *argv[] )
{
	int drive = 0;
	int track, sector, rate, density;
	int t;
	char *dev="/dev/fd0";
	
	while((t=getopt(argc, argv, "c0123h?"))!=-1)
	{
		switch(t)
		{
			case '0':
				drive = 0;
				dev="/dev/fd0";
				break;
			
			case '1':
				drive = 1;
				dev="/dev/fd1";
				break;
			case '2':
				drive = 2;
				dev="/dev/fd2";
				break;
			
			case '3':
				drive = 3;
				dev="/dev/fd3";
				break;
			
			case 'c':
				nosync=1;
				break;
			
			default:
				usage();
				
		}
	}
				
	for(rate=0; rate<3; rate++)
	{
		for(track=0; track<2; track++)
		{
			for(sector=0; sector<2; sector++)
			{
			  for(density=0; density<1; density++)
			  {
				docmd(drive, track, sector, rate, density, dev);
				testoutput(drive, track, sector, rate, density);
			  }
			}
		}
	}
}

void
usage(void)
{
	int t;
	static char *msg[]=
	{
		"\nUsage: os9guess [options]",
		"Function: tries to figure out os9 disk format",
		"Options:",
		"  -0, -1, -2, -3 set drive number",
		"  -c             don't check sync bytes in header",
		"",
		"Another fine program from Bob van der Poel Software <bvdpoel@uniserve.com>",
		""
	};
	
	for(t=0; t<sizeof(msg)/sizeof(msg[0]); fprintf(stderr, "%s\n", msg[t++]));
	exit(1);

}

void
testoutput(int drive, int track, int sector, int rate, int density)
{
	FILE *fd;
	u_char buff[256];
	int t;
	int totsect, sects, tracks,  numsides, sectrk;
	
	fd=fopen(sfile, "r");
	
	if(fd==NULL) return;
	
	if(fread(buff, sizeof(buff), 1, fd) != 1 ) return;
	
	// test DD_SYNC. If not valid, try some more
	
	if(nosync==0 && 
		( buff[0x60]!=0x43 || buff[0x61]!=0x72 ||
		  buff[0x62]!=0x75 || buff[0x63]!=0x7a) )
		  return;
	
	// we have a valid lsn0. Print the info
	
	
	printf("Drive: %d\n", drive);
	printf("Data rate: %d\n", rate);
	printf("First track: %d\n", track);
	printf("First sector: %d\n", sector);
	printf("Density Track0/Side0: %s\n", density ? "double (MFM)" : "single (FM)");
	totsect= (buff[0]<<16) + (buff[1]<<8) + (buff[2]);
	printf("Total Sectors: %d\n", totsect);
	sectrk = buff[3];
	printf("Sectors per track: %d\n", sectrk);
	printf("Sector count: %d\n", buff[0x11]*0x100 + buff[0x12]);
	numsides=buff[0x10] & 1 ? 2 : 1;
	printf("Sides: %d\n",  numsides);
	if(numsides && sectrk && totsect) {
		tracks = totsect / numsides / sectrk;
		sects = totsect - tracks * numsides * sectrk - (numsides-1) * sectrk;;
		if (sects > 0) { 
			printf("Track0 sectors: %d\n", sects); 
			tracks++;
		}
		printf("Tracks/side: %d\n", tracks);
	}
	else printf("Tracks/side: UNKNOWN\n");
	
	printf("Density: %s\n", buff[0x10] & 2 ? "Double" : "Single");
	printf("Track density %s\n", buff[0x10] & 4 ? "96 tpi" : "48 tpi");
	printf("Disk Name: ");
	for(t=0x1f; ; t++)
	{
		if(buff[t]==0)
		{
			printf("\n");
			break;
		}
		else if(buff[t]>=0x80)
		{
			printf("%c\n", buff[t]-0x80);
			break;
		}
		else printf("%c", buff[t]);
	}
	
	printf("\nContinue [Yes]:");
	fgets(buff, sizeof(buff)-10, stdin);
	
	if(buff[0]=='n' || buff[0]=='N') exit(0);
}

void
docmd(int drive, int track, int sector, int rate, int density, char *dev)
{
	char cmd[256];
	sprintf(cmd, "floppycontrol --resetnow 0 %s " , dev );
	puts(cmd);
	system(cmd);
	sprintf(cmd, "fdrawcmd read %d %d  0 %d 1  1 0x1b 0xff %s rate=%d  need_seek  length=256 track=%d  %s >%s", drive, track, sector, density ? "" : "fm", rate, track, dev, sfile);
	
	puts(cmd);
	system(cmd);
}

